﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace TestZoom
{
    /// <summary>
    /// UserControl zur Darstellung
    /// eines Zoom-Bildes
    /// </summary>
    [System.ComponentModel.DesignerCategory("Code")]
    public partial class SelectionPictureBox
        : PictureBox
    {
        #region Properties

        /// <summary>
        /// vom aufrufenden Fenster eingestellte Zoomstufe,
        /// 0: Vollbild
        /// 1 ... n: Zoom
        /// zur korrekten Darstellung des aktuellen Bildes
        /// </summary>
        [Browsable(false)]
        internal int Zoom { get; set; }

        /// <summary>
        /// Kennung für "Pause"-Ausschrift im Kamerabild
        /// </summary>
        [Browsable(false)]
        internal bool Pause { get; set; }

        #endregion Properties

        #region Private Member

        /// <summary>
        /// Font zur Darstellung von "Pause"
        /// </summary>
        private Font MyFont;

        /// <summary>
        /// Breite des Zoom-Bildes
        /// </summary>
        private int ImgWidth;

        /// <summary>
        /// Höhe des Zoom-Bildes
        /// </summary>
        private int ImgHeight;

        #endregion Private Member

        #region Konstruktor

        /// <summary>
        /// Standardkonstruktor der Klasse SelectionPictureBox
        /// </summary>
        public SelectionPictureBox()
        {
            this.InitializeComponent();
            this.Pause = false;
            this.Zoom = 0;
            this.MyFont = new Font("Arial", 20, FontStyle.Bold);
        }

        #endregion Konstruktor

        #region Public Stuff

        /// <summary>
        /// Übergabe des aktuellen Bildes
        /// </summary>
        /// <param name="bmp">das Bild</param>
        internal void SetImage(Bitmap bmp)
        {
            if (bmp == null)
            {
                this.ImgWidth = 1;
                this.ImgHeight = 1;
            }
            else
            {
                this.ImgWidth = bmp.Width;
                this.ImgHeight = bmp.Height;
            }

            this.Image = bmp;
        }

        #endregion Public Stuff

        #region Scaling

        /// <summary>
        /// Umrechnung einer View-Koordinate
        /// in die entsprechende Pixel-Koordinate
        /// in Abhängigkeit vom SizeMode
        /// </summary>
        /// <param name="pt">die View-Koordinate</param>
        /// <returns>die Pixel-Koordinate</returns>
        internal Point ScaleToImage(Point pt)
        {
            if (this.Image == null)
            {
                return Point.Empty;
            }

            double zoom = 1;
            if (this.Zoom > 1)
            {
                zoom = this.Zoom;
            }

            double xFaktor, yFaktor, faktor;

            xFaktor = (double)this.Width / (double)this.ImgWidth * zoom;
            yFaktor = (double)this.Height / (double)this.ImgHeight * zoom;
            faktor = Math.Min(xFaktor, yFaktor);

            Point realPoint = Point.Empty;
            switch (this.SizeMode)
            {
                case PictureBoxSizeMode.AutoSize:
                case PictureBoxSizeMode.Zoom:

                    double offX = 0;
                    double offY = 0;
                    if (this.Zoom == 0)
                    {
                        // Rand berücksichtigen
                        if (xFaktor > yFaktor)
                        {
                            offX = ((double)this.Width - ((double)this.ImgWidth * faktor)) / 2d;
                        }
                        else
                        {
                            offY = ((double)this.Height - ((double)this.ImgHeight * faktor)) / 2d;
                        }
                    }

                    realPoint.X = (int)(((double)pt.X - offX) / faktor);
                    realPoint.Y = (int)(((double)pt.Y - offY) / faktor);
                    break;
                default:
                    Console.Out.WriteLine("PictureBox.SizeMode nicht unterstützt!");
                    break;
            }

            return realPoint;
        }

        #endregion Scaling

        #region Protected Override

        /// <summary>
        /// Darstellung des Kamerabildes
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if (this.Image == null)
            {
                return;
            }

            if (this.Pause)
            {
                Panel parent = this.Parent as Panel;
                Point pt1 = new Point(50, 50);
                if (parent != null)
                {
                    // TODO: die Scroll-Position draufaddieren
                    pt1.Y += parent.AutoScrollPosition.Y;
                }
                e.Graphics.DrawString("Pause", this.MyFont, Brushes.Red, pt1);
            }
        }

        #endregion Protected Override
    }
}
